@@ -6,7 +6,7 @@ import numpy as np
 from typing import Callable, Union, Dict, List, Tuple, Sequence, Any
 from .utils import Utils
 from .transformer import TransformerBase, EstimatorBase
-from .model import add_ranks
+from .model import add_ranks, coerce_dataframe_types
 import deprecation
 import ir_measures
 from ir_measures.measures import BaseMeasure 
@@ -56,15 +56,15 @@ _irmeasures_columns = {
 }
 
 def _convert_measures(metrics : MEASURES_TYPE) -> Tuple[Sequence[BaseMeasure], Dict[BaseMeasure,str]]:
-    from ir_measures import convert_trec_name
+    from ir_measures import parse_trec_measure
     rtr = []
     rev_mapping = {}
     for m in metrics:
         if isinstance(m, BaseMeasure):
             rtr.append(m)
             continue
-        if isinstance(m, str):
-            measures = convert_trec_name(m)
+        elif isinstance(m, str):
+            measures = parse_trec_measure(m)
             if len(measures) == 1:
                 metric = measures[0]
                 rtr.append(metric)
@@ -74,7 +74,9 @@ def _convert_measures(metrics : MEASURES_TYPE) -> Tuple[Sequence[BaseMeasure], D
                 rtr.extend(measures)
             else:
                 raise KeyError("Could not convert measure %s" % m)
-    assert len(rtr) > 0
+        else:
+            raise KeyError("Unknown measure %s of type %s" % (str(m), str(type(m))))
+    assert len(rtr) > 0, "No measures were found in %s" % (str(metrics))
     return rtr, rev_mapping
 
 #list(iter_calc([ir_measures.AP], qrels, run))
@@ -84,7 +86,8 @@ def _ir_measures_to_dict(
         metrics: Sequence[BaseMeasure],
         rev_mapping : Dict[BaseMeasure,str], 
         num_q : int,
-        perquery : bool = True):
+        perquery : bool = True,
+        backfill_qids : Sequence[str] = None):
     from collections import defaultdict
     if perquery:
         # qid -> measure -> value
@@ -93,7 +96,20 @@ def _ir_measures_to_dict(
             metric = m.measure
             metric = rev_mapping.get(metric, str(metric))
             rtr[m.query_id][metric] = m.value
+        # When reporting per-query results, it can desirable to show something for topics that were executed
+        # do not have corresponding qrels. If the caller passes in backfill_qids, we'll ensure that these
+        # qids are present, and if not add placeholders with NaN values for all measures.
+        if backfill_qids is not None:
+            backfill_count = 0
+            for qid in backfill_qids:
+                if qid not in rtr:
+                    backfill_count += 1
+                    for metric in metrics:
+                        rtr[qid][rev_mapping.get(metric, str(metric))] = float('NaN')
+            if backfill_count > 0:
+                warn(f'{backfill_count} topic(s) not found in qrels. Scores for these topics are given as NaN and should not contribute to averages.')
         return rtr
+    assert backfill_qids is None, "backfill_qids only supported when perquery=True"
     # measure -> value
     rtr = {rev_mapping.get(m, str(m)): m.aggregator() for m in metrics}
     for m in seq:
@@ -110,7 +126,8 @@ def _run_and_evaluate(
         qrels: pd.DataFrame, 
         metrics : MEASURES_TYPE, 
         perquery : bool = False,
-        batch_size = None):
+        batch_size = None,
+        backfill_qids : Sequence[str] = None):
     
     metrics, rev_mapping = _convert_measures(metrics)
     qrels = qrels.rename(columns={'qid': 'query_id', 'docno': 'doc_id', 'label': 'relevance'})
@@ -120,12 +137,16 @@ def _run_and_evaluate(
     # if its a DataFrame, use it as the results
     if isinstance(system, pd.DataFrame):
         res = system
+        res = coerce_dataframe_types(res)
+        if len(res) == 0:
+            raise ValueError("%d topics, but no results in dataframe" % len(topics))
         evalMeasuresDict = _ir_measures_to_dict(
             ir_measures.iter_calc(metrics, qrels, res.rename(columns=_irmeasures_columns)), 
             metrics,
             rev_mapping,
             num_q,
-            perquery)
+            perquery,
+            backfill_qids)
 
     elif batch_size is None:
         #transformer, evaluate all queries at once
@@ -134,30 +155,55 @@ def _run_and_evaluate(
         res = system.transform(topics)
         endtime = timer()
         runtime =  (endtime - starttime) * 1000.
+
+        res = coerce_dataframe_types(res)
+
+        if len(res) == 0:
+            raise ValueError("%d topics, but no results received from %s" % (len(topics), str(system)) )
+
         evalMeasuresDict = _ir_measures_to_dict(
             ir_measures.iter_calc(metrics, qrels, res.rename(columns=_irmeasures_columns)), 
             metrics,
             rev_mapping,
             num_q,
-            perquery)
+            perquery,
+            backfill_qids)
     else:
         #transformer, evaluate queries in batches
         assert batch_size > 0
         starttime = timer()
-        results=[]
-        evalMeasuresDict={}
-        for res in system.transform_gen(topics, batch_size=batch_size):
+        evalMeasuresDict = {}
+        remaining_qrel_qids = set(qrels.query_id)
+        for i, (res, batch_topics) in enumerate( system.transform_gen(topics, batch_size=batch_size, output_topics=True)):
+            if len(res) == 0:
+                raise ValueError("batch of %d topics, but no results received in batch %d from %s" % (len(batch_topics), i, str(system) ) )
             endtime = timer()
             runtime += (endtime - starttime) * 1000.
-            localEvalDict = _ir_measures_to_dict(
-                ir_measures.iter_calc(metrics, qrels, res.rename(columns=_irmeasures_columns)),
+            res = coerce_dataframe_types(res)
+            batch_qids = set(batch_topics.qid)
+            batch_qrels = qrels[qrels.query_id.isin(batch_qids)] # filter qrels down to just the qids that appear in this batch
+            remaining_qrel_qids.difference_update(batch_qids)
+            batch_backfill = [qid for qid in backfill_qids if qid in batch_qids] if backfill_qids is not None else None
+            evalMeasuresDict.update(_ir_measures_to_dict(
+                ir_measures.iter_calc(metrics, batch_qrels, res.rename(columns=_irmeasures_columns)),
                 metrics,
                 rev_mapping,
                 num_q,
-                True)
-            evalMeasuresDict.update(localEvalDict)
+                perquery=True,
+                backfill_qids=batch_backfill))
             starttime = timer()
+        if remaining_qrel_qids:
+            # there are some qids in the qrels that were not in the topics. Get the default values for these and update evalMeasuresDict
+            missing_qrels = qrels[qrels.query_id.isin(remaining_qrel_qids)]
+            empty_res = pd.DataFrame([], columns=['query_id', 'doc_id', 'score'])
+            evalMeasuresDict.update(_ir_measures_to_dict(
+                ir_measures.iter_calc(metrics, missing_qrels, empty_res),
+                metrics,
+                rev_mapping,
+                num_q,
+                perquery=True))
         if not perquery:
+            # aggregate measures if not in per query mode
             aggregators = {rev_mapping.get(m, str(m)): m.aggregator() for m in metrics}
             for q in evalMeasuresDict:
                 for metric in metrics:
@@ -178,13 +224,15 @@ def Experiment(
         perquery : bool = False,
         dataframe : bool =True,
         batch_size : int = None,
-        drop_unused : bool = False,
+        filter_by_qrels : bool = False,
+        filter_by_topics : bool = True,
         baseline : int = None,
         test : Union[str,TEST_FN_TYPE] = "t",
         correction : str = None,
         correction_alpha : float = 0.05,
         highlight : str = None,
-        round : Union[int,Dict[str,int]] = None):
+        round : Union[int,Dict[str,int]] = None,
+        **kwargs):
     """
     Allows easy comparison of multiple retrieval transformer pipelines using a common set of topics, and
     identical evaluation measures computed using the same qrels. In essence, each transformer is applied on 
@@ -203,7 +251,8 @@ def Experiment(
         batch_size(int): If not None, evaluation is conducted in batches of batch_size topics. Default=None, which evaluates all topics at once. 
             Applying a batch_size is useful if you have large numbers of topics, and/or if your pipeline requires large amounts of temporary memory
             during a run.
-        drop_unused(bool): If True, will drop topics from the topics dataframe that have qids not appearing in the qrels dataframe. 
+        filter_by_qrels(bool): If True, will drop topics from the topics dataframe that have qids not appearing in the qrels dataframe. 
+        filter_by_topics(bool): If True, will drop topics from the qrels dataframe that have qids not appearing in the topics dataframe. 
         perquery(bool): If True return each metric for each query, else return mean metrics across all queries. Default=False.
         dataframe(bool): If True return results as a dataframe. Else as a dictionary of dictionaries. Default=True.
         baseline(int): If set to the index of an item of the retr_system list, will calculate the number of queries 
@@ -241,6 +290,10 @@ def Experiment(
         warn_old_sig = True
     if warn_old_sig:
         warn("Signature of Experiment() is now (retr_systems, topics, qrels, eval_metrics), please update your code", DeprecationWarning, 2)
+
+    if 'drop_unused' in kwargs:
+        filter_by_qrels = kwargs.pop('drop_unused')
+        warn('drop_unused is deprecated; use filter_by_qrels instead')
     
     if baseline is not None:
         assert int(baseline) >= 0 and int(baseline) < len(retr_systems)
@@ -275,10 +328,18 @@ def Experiment(
         return value
 
     # drop queries not appear in the qrels
-    if drop_unused:
+    if filter_by_qrels:
         # the commented variant would drop queries not having any RELEVANT labels
         # topics = topics.merge(qrels[qrels["label"] > 0][["qid"]].drop_duplicates())        
         topics = topics.merge(qrels[["qid"]].drop_duplicates())
+        if len(topics) == 0:
+            raise ValueError('There is no overlap between the qids found in the topics and qrels. If this is intentional, set filter_by_topics=False and filter_by_qrels=False.')
+
+    # drop qrels not appear in the topics
+    if filter_by_topics:
+        qrels = qrels.merge(topics[["qid"]].drop_duplicates())
+        if len(qrels) == 0:
+            raise ValueError('There is no overlap between the qids found in the topics and qrels. If this is intentional, set filter_by_topics=False and filter_by_qrels=False.')
 
     from scipy import stats
     if test == "t":
@@ -292,7 +353,7 @@ def Experiment(
     elif len(names) != len(retr_systems):
         raise ValueError("names should be the same length as retr_systems")
 
-    all_qids = topics["qid"].values
+    all_topic_qids = topics["qid"].values
 
     evalsRows=[]
     evalDict={}
@@ -305,24 +366,14 @@ def Experiment(
 
     # run and evaluate each system
     for name,system in zip(names, retr_systems):
-        time, evalMeasuresDict = _run_and_evaluate(system, topics, qrels, eval_metrics, perquery=perquery or baseline is not None, batch_size=batch_size)
-        
-        if perquery or baseline is not None:
-            # this ensures that all queries are present in various dictionaries
-            # its equivalent to "trec_eval -c"
-            (evalMeasuresDict, missing) = Utils.ensure(evalMeasuresDict, eval_metrics, all_qids)
-            if missing > 0:
-                warn("%s was missing %d queries, expected %d" % (name, missing, len(all_qids) ))
+        time, evalMeasuresDict = _run_and_evaluate(system, topics, qrels, eval_metrics, perquery=perquery or baseline is not None, batch_size=batch_size, backfill_qids=all_topic_qids if perquery else None)
 
         if baseline is not None:
             evalDictsPerQ.append(evalMeasuresDict)
             evalMeasuresDict = Utils.mean_of_measures(evalMeasuresDict)
 
-        if mrt_needed:
-            evalMeasuresDict["mrt"] = time / float(len(all_qids))
-
         if perquery:
-            for qid in all_qids:
+            for qid in evalMeasuresDict:
                 for measurename in evalMeasuresDict[qid]:
                     evalsRows.append([
                         name, 
@@ -336,6 +387,8 @@ def Experiment(
             evalDict[name] = evalMeasuresDict
         else:
             import builtins
+            if mrt_needed:
+                evalMeasuresDict["mrt"] = time / float(len(all_topic_qids))
             actual_metric_names = list(evalMeasuresDict.keys())
             # gather mean values, applying rounding if necessary
             evalMeasures=[ _apply_round(m, evalMeasuresDict[m]) for m in actual_metric_names]
@@ -345,7 +398,7 @@ def Experiment(
 
     if dataframe:
         if perquery:
-            df = pd.DataFrame(evalsRows, columns=["name", "qid", "measure", "value"])
+            df = pd.DataFrame(evalsRows, columns=["name", "qid", "measure", "value"]).sort_values(['name', 'qid'])
             if round is not None:
                 df["value"] = df["value"].round(round)
             return df
@@ -454,17 +507,17 @@ def KFoldGridSearch(
     been executed.
 
     Args:
-        - pipeline(TransformerBase): a transformer or pipeline to tune
-        - params(dict): a two-level dictionary, mapping transformer to param name to a list of values
-        - topics_list(List[DataFrame]): a *list* of topics dataframes to tune upon
-        - qrels(DataFrame or List[DataFrame]): qrels to tune upon. A single dataframe, or a list for each fold.       
-        - metric(str): name of the metric on which to determine the most effective setting. Defaults to "map".
-        - batch_size(int): If not None, evaluation is conducted in batches of batch_size topics. Default=None, which evaluates all topics at once. 
+        pipeline(TransformerBase): a transformer or pipeline to tune
+        params(dict): a two-level dictionary, mapping transformer to param name to a list of values
+        topics_list(List[DataFrame]): a *list* of topics dataframes to tune upon
+        qrels(DataFrame or List[DataFrame]): qrels to tune upon. A single dataframe, or a list for each fold.       
+        metric(str): name of the metric on which to determine the most effective setting. Defaults to "map".
+        batch_size(int): If not None, evaluation is conducted in batches of batch_size topics. Default=None, which evaluates all topics at once. 
             Applying a batch_size is useful if you have large numbers of topics, and/or if your pipeline requires large amounts of temporary memory
             during a run. Default is None.
-        - jobs(int): Number of parallel jobs to run. Default is 1, which means sequentially.
-        - backend(str): Parallelisation backend to use. Defaults to "joblib". 
-        - verbose(bool): whether to display progress bars or not
+        jobs(int): Number of parallel jobs to run. Default is 1, which means sequentially.
+        backend(str): Parallelisation backend to use. Defaults to "joblib". 
+        verbose(bool): whether to display progress bars or not
 
     Returns:
     A tuple containing, firstly, the results of pipeline on the test topics after tuning, and secondly, a list of the best parameter settings for each fold.
@@ -549,23 +602,26 @@ def GridSearch(
     topics and qrels, and for the specified measure.
 
     Args:
-        - pipeline(TransformerBase): a transformer or pipeline to tune
-        - params(dict): a two-level dictionary, mapping transformer to param name to a list of values
-        - topics(DataFrame): topics to tune upon
-        - qrels(DataFrame): qrels to tune upon       
-        - metric(str): name of the metric on which to determine the most effective setting. Defaults to "map".
-        - batch_size(int): If not None, evaluation is conducted in batches of batch_size topics. Default=None, which evaluates all topics at once. 
+        pipeline(TransformerBase): a transformer or pipeline to tune
+        params(dict): a two-level dictionary, mapping transformer to param name to a list of values
+        topics(DataFrame): topics to tune upon
+        qrels(DataFrame): qrels to tune upon       
+        metric(str): name of the metric on which to determine the most effective setting. Defaults to "map".
+        batch_size(int): If not None, evaluation is conducted in batches of batch_size topics. Default=None, which evaluates all topics at once. 
             Applying a batch_size is useful if you have large numbers of topics, and/or if your pipeline requires large amounts of temporary memory
             during a run. Default is None.
-        - jobs(int): Number of parallel jobs to run. Default is 1, which means sequentially.
-        - backend(str): Parallelisation backend to use. Defaults to "joblib". 
-        - verbose(bool): whether to display progress bars or not
-        - return_type(str): whether to return the same transformer with optimal pipeline setting, and/or a setting of the
+        jobs(int): Number of parallel jobs to run. Default is 1, which means sequentially.
+        backend(str): Parallelisation backend to use. Defaults to "joblib". 
+        verbose(bool): whether to display progress bars or not
+        return_type(str): whether to return the same transformer with optimal pipeline setting, and/or a setting of the
             higher metric value, and the resulting transformers and settings.
     """
     # save state
     initial_state = _save_state(params)
 
+    if isinstance(metric, list):
+        raise KeyError("GridSearch can only maximise ONE metric, but you passed a list (%s)." % str(metric))
+
     grid_outcomes = GridScan(
         pipeline, 
         params, 
@@ -621,29 +677,29 @@ def GridScan(
     as well as controls in the case of BatchRetrieve.
 
     Args:
-        - pipeline(TransformerBase): a transformer or pipeline
-        - params(dict): a two-level dictionary, mapping transformer to param name to a list of values
-        - topics(DataFrame): topics to tune upon
-        - qrels(DataFrame): qrels to tune upon       
-        - metrics(List[str]): name of the metrics to report for each setting. Defaults to ["map"].
-        - batch_size(int): If not None, evaluation is conducted in batches of batch_size topics. Default=None, which evaluates all topics at once. 
+        pipeline(TransformerBase): a transformer or pipeline
+        params(dict): a two-level dictionary, mapping transformer to param name to a list of values
+        topics(DataFrame): topics to tune upon
+        qrels(DataFrame): qrels to tune upon       
+        metrics(List[str]): name of the metrics to report for each setting. Defaults to ["map"].
+        batch_size(int): If not None, evaluation is conducted in batches of batch_size topics. Default=None, which evaluates all topics at once. 
             Applying a batch_size is useful if you have large numbers of topics, and/or if your pipeline requires large amounts of temporary memory
             during a run. Default is None.
-        - jobs(int): Number of parallel jobs to run. Default is 1, which means sequentially.
-        - backend(str): Parallelisation backend to use. Defaults to "joblib". 
-        - verbose(bool): whether to display progress bars or not
-        - dataframe(bool): return a dataframe or a list
+        jobs(int): Number of parallel jobs to run. Default is 1, which means sequentially.
+        backend(str): Parallelisation backend to use. Defaults to "joblib". 
+        verbose(bool): whether to display progress bars or not
+        dataframe(bool): return a dataframe or a list
     Returns:
-        - A dataframe showing the effectiveness of all evaluated settings, if dataframe=True
-        - A list of settings and resulting evaluation measures, if dataframe=False
+        A dataframe showing the effectiveness of all evaluated settings, if dataframe=True
+        A list of settings and resulting evaluation measures, if dataframe=False
     Raises:
-        - ValueError: if a specified transformer does not have such a parameter
+        ValueError: if a specified transformer does not have such a parameter
 
     Example::
 
         # graph how PL2's c parameter affects MAP
         pl2 = pt.BatchRetrieve(index, wmodel="PL2", controls={'c' : 1})
-        rtr = pt.GridSearch(
+        rtr = pt.GridScan(
             pl2, 
             {pl2 : {'c' : [0.1, 1, 5, 10, 20, 100]}}, 
             topics,
